# Convert a Selenium RSpec to Selenium Pytest project
# 
# 
# /Users/zhimin/work/projects/agiletravel-ui-tests/selenium-webdriver-rspec

# TODO 
# - try_for
# - "#{}"

spec_files = []
helper_files = []
page_files = []

@page_class_hash = {}

require 'active_support/all'

def convert_page_file(page_file_path, target_project_dir)
  
  source_page_content = File.read(page_file_path)
  class_name = nil
  result = []
  result << "from pages.abstract_page import AbstractPage"
  if source_page_content.include?("sleep")
    result << "import time"
  end
  result << ""
  result << "from selenium import webdriver"
  result << "from selenium.webdriver.support.ui import Select"
  result << "from selenium.webdriver.common.by import By"
  result << "from selenium.webdriver.common.keys import Keys"
  result << ""
  
  
  flag_found_page_class_line = false
  
  File.readlines(page_file_path).each do |rb_line|
    py_line = rb_line.rstrip
    
    if py_line =~ /^s*class\s*(\w+)\s*<\s*AbstractPage/
      class_name = $1
      py_line = "class #{$1}(AbstractPage):"
      flag_found_page_class_line = true
      result << py_line
      next
    end

    next unless flag_found_page_class_line

    if py_line =~ /^(\s*)def\s+initialize\(driver\)\s*$/
      py_line = "#{$1}def __init__(self, driver):"
      result << py_line
      next
    end
      
    # "super(driver, \"\") # <= TEXT UNIQUE TO THIS PAGE"
    if py_line =~ /^(\s*)super\(driver\s*/
      py_line = "#{$1}super().__init__(driver)"
      result << py_line
      next
    end
    
    if py_line =~ /^(\s*)def\s+(\w+)\s*\((.*)\)/
      py_line = "#{$1}def #{$2}(self, #{$3}):"
      result << py_line    
      next;
    elsif py_line =~ /^(\s*)def\s+(\w+)(.*)/
      if $3.strip.empty?
        py_line = "#{$1}def #{$2}(self):"
      else
        py_line = "#{$1}def #{$2}(self, #{$3}):"
      end
      result << py_line    
      next;
    end
    
    if py_line =~  /^(\s*)end\s*$/
      space_in_front = $1      
      result << "#{space_in_front}"
      next
    end

    if py_line =~  /^(\s*)if(.*)/
      space_in_front = $1      
      result << "#{space_in_front}if#{$2}:"
      next
    end

    if py_line =~  /^(\s*)else\s*$/
      space_in_front = $1      
      result << "#{space_in_front}else:"
      next
    end

    if py_line =~ /^(\s*)driver.get(.*)/
      space_in_front = $1      
      result << "#{space_in_front}self.driver.get#{$2}"
      next
    end
    
    if py_line =~ /^(\s*)sleep\s+([\d\.]+)$/
      space_in_front = $1      
      py_line = "#{space_in_front}time.sleep(#{$2})"
      result << py_line     
      next    
    end
    if py_line =~ /^(\s*)sleep\s+([\d\.]+)\s+#/
      space_in_front = $1      
      py_line = "#{space_in_front}time.sleep(#{$2})"
      result << py_line     
      next    
    end
    
    if py_line.strip == ""
      result << ""
      next
    end
    
    py_line.gsub!(":control", "Keys.CONTROL")
    py_line.gsub!(":end", "Keys.END")
    py_line.gsub!(":down", "Keys.DOWN")
    py_line.gsub!(":up", "Keys.UP")
    py_line.gsub!(":return", "Keys.RETURN")
    py_line.gsub!(":shift", "Keys.SHIIFT")
    py_line.gsub!(":backspace", "Keys.BACKSPACE")
    py_line.gsub!(":tab", "Keys.TAB")
    
    py_line.gsub!("driver.find_element(:name,", "self.driver.find_element(By.NAME,")
    py_line.gsub!("driver.find_element(:id,", "self.driver.find_element(By.ID,")
    py_line.gsub!("driver.find_element(:xpath,", "self.driver.find_element(By.XPATH,")
    py_line.gsub!("driver.find_element(:link_text,", "self.driver.find_element(By.LINK_TEXT,")
    py_line.gsub!("driver.find_element(:partial_link_text,", "self.driver.find_element(By.PARTIAL_LINK_TEXT,")
    py_line.gsub!("driver.find_element(:tag_name,", "self.driver.find_element(By.TAG_NAME,")
    py_line.gsub!("driver.find_element(:class_name,", "self.driver.find_element(By.CLASS_NAME,")
    py_line.gsub!("driver.find_element(:class,", "self.driver.find_element(By.CLASS_NAME,")
    py_line.gsub!("driver.find_element(:css,", "self.driver.find_element(By.CSS_SELECTOR,")
     
    py_line.gsub!(").select_by(:text, ", ").select_by_visible_text(")
    py_line.gsub!("Selenium::WebDriver::Support::Select.new(", "Select(")
      
    # ENV["BROWSER"]  
    # os.environ.get('BROWSER')
    
    if py_line.end_with?(".click") || py_line.end_with?(".submit")
      py_line += "()"
    end
        
    result << py_line    
  end
    
  target_page_content = result.join("\n")
  # puts target_page_content

  pages_dir = File.join( target_project_dir, "pages")
  FileUtils.mkdir_p(pages_dir) unless Dir.exist?(pages_dir)
  target_file = File.join(pages_dir, File.basename(page_file_path).gsub(".rb", ".py"))
  File.open(target_file, "w").write(target_page_content)
  
  # class LoginPage < AbstractPage  
  # class LoginPage(AbstractPage):
  
  # remove initialize(driver)
  return class_name
end


def convert_test_file(test_file_path, target_project_test_dir)
  
  source_page_content = File.read(test_file_path)  
  result = []

  result << "import unittest"
  result << "import xmlrunner"
  result << "import time"
  result << "import datetime"
  result << "import sys"
  result << "import os"
  result << ""
  result << "from selenium import webdriver"
  result << "from selenium.webdriver.support.ui import Select"
  result << "from selenium.webdriver.common.by import By"
  result << "from selenium.webdriver.support.ui import WebDriverWait"
  result << "from selenium.webdriver.support import expected_conditions as EC"
  result << "from selenium.webdriver.common.keys import Keys"
  result << ""
  
  result << "# load modules from parent dir, pages will be referred from there too."
  result << 'sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../")'
  result << "from test_helper import TestHelper"
  result << ""
  
  @page_class_hash.each do |key, val|
    result << "from pages.#{key} import #{val}"
  end  
  result << ""
  
  #describe "Select Flights" do
  #  include TestHelper
    
  # class FlightTestCase(unittest.TestCase, TestHelper):
  
  
  #   it "[3] Return trip" do
  # def test_return_trip(self):

  
  # before(:all) do
      
  # @classmethod
  # def setUpClass(cls):

  
  # after (:all) do  

  # @classmethod
  # def tearDownClass(cls):
  
  
  # before(:each) do  
  
  # def setUp(self):
  
  
  # sleep 1
  # sleep(2)
  
  # time.sleep(1)


  File.readlines(test_file_path).each do |rb_line|
    py_line = rb_line.rstrip
    
    if py_line =~ /^(\s*)describe\s+['"](.+)['"]\s+do\s*$/
      space_in_front = $1
      test_suite_name = $2
      test_class_name = test_suite_name.gsub("(", "").gsub(")", "").gsub(" ", "").classify
      result << "" 
      py_line = "class " + test_class_name  + "TestCase(unittest.TestCase, TestHelper):"
      result << py_line 
      next
    end
      
    #describe "Select Flights" do
    #  include TestHelper
    
    # class FlightTestCase(unittest.TestCase, TestHelper):
  
    if py_line =~ /^(\s*)@?driver\s*=\s*\$driver\s*=\s*Selenium::WebDriver\.for/ ||
       py_line =~ /^(\s*)@?driver\s*=\s*Selenium::WebDriver\.for/
      space_in_front = $1
      py_line = space_in_front + "self.driver = self.open_browser();"
      result << py_line     
      next
    end
    
    if py_line =~ /^(\s*)driver\.get\(site_url\)\s*$/
      space_in_front = $1
      py_line = space_in_front + "self.driver.get(self.site_url())"
      result << py_line     
      next
    end
    
    if py_line =~ /^(\s*)driver\.navigate\.to\(site_url\)\s*$/
      space_in_front = $1
      py_line = space_in_front + "self.driver.get(self.site_url())"
      result << py_line     
      next
    end
    
    if py_line =~ /^(\s*)driver\.quit\s+unless\s+debugging\?\s*$/
      space_in_front = $1
      py_line = space_in_front + "if not self.is_debugging():"
      result << py_line     
      py_line = space_in_front + "  " + "self.driver.quit()"
      result << py_line     
      next
    end
    
    
    if py_line =~ /^(\s*)before\s*\(\s*:each\s*\)\s+do\s*$/
      result << "" 
      py_line = "#{$1}def setUp(self):"
      result << py_line    
      next;
    elsif py_line =~ /^(\s*)before\s*\(\s*:all\s*\)\s+do\s*$/
      result << "" 
      result << "#{$1}@classmethod"
      result << "#{$1}def setUpClass(self):"
      next;      
    elsif py_line =~ /^(\s*)after\s*\(\s*:all\s*\)\s+do\s*$/ 
      result << "" 
      result << "#{$1}@classmethod"
      result << "#{$1}def tearDownClass(self):"
      next;      
    elsif py_line =~ /^(\s*)after\s*\(\s*:each\s*\)\s+do\s*$/ 
      py_line = "#{$1}def tearDown(self):"
      result << "" 
      result << py_line    
      next;      
    end
    
    if py_line.strip == "include TestHelper"
      next
    end
    
    if py_line =~ /^(\s*)driver\.manage\(\)\.window\(\)\.resize_to(.*)/
      py_line = "#{$1}self.driver.set_window_size" + $2
      result << py_line    
      next;              
    end

    if py_line =~ /^(\s*)driver\.manage\(\)\.window\(\)\.move_to(.*)/
      py_line = "#{$1}self.driver.set_window_position" + $2
      result << py_line    
      next;              
    end
    
    
    if py_line.strip == "load File.dirname(__FILE__) + '/../test_helper.rb'"
      next
    end
    
    if py_line =~  /^(\s*)if(.*)/
      space_in_front = $1      
      result << "#{space_in_front}if#{$2}:"
      next
    end

    if py_line =~  /^(\s*)else\s*$/
      space_in_front = $1      
      result << "#{space_in_front}else:"
      next
    end

    if py_line =~ /^(\s*)driver.get(.*)/
      space_in_front = $1      
      result << "#{space_in_front}self.driver.get#{$2}"
      next
    end
    
    if py_line =~  /^(\s*)end\s*$/
      space_in_front = $1      
      result << "#{space_in_front}"
      next
    end

    if py_line.strip == ""
      result << ""
      next
    end
        
        
    # it "[3] Return trip" do => def test_return_trip(self):
    
    if py_line =~ /^(\s*)it\s+(.*),\s+:tag/ || py_line =~ /^(\s*)it\s+(.*)\s+do\s*$/
      space_in_front = $1
      test_name = $2.strip
      test_name = test_name.gsub("'", "").gsub('"', "").gsub("(", "").gsub(")", "")
      test_name.strip!
      if test_name =~ /\[.+\]\s*(.*)/
        test_name = $1.strip
      end
      test_name = test_name.gsub(" ", "_").gsub("-", "_")      
      test_name = test_name.downcase            
      py_line = "#{space_in_front}def test_#{test_name}(self):"
      result << ""
      result << py_line    
      next;
    end
    
      
    # .new(driver) => (self.driver)
    py_line.gsub!(".new(driver)", "(self.driver)")
    
    py_line.gsub!("page_text", "self.page_text()")
    py_line.gsub!("driver.page_source", "self.driver.page_source")
    
  
    
    # passenger_page.click_next => passenger_page.click_next()
    if py_line =~ /^(\s*)(.*)\.click(\w+)\s*$/
      py_line += "()"
      result << py_line     
      next           
    end
    
    if py_line.strip.empty?
      next
    end
    
    if py_line =~ /^(\s*)sleep\s+([\d\.]+)$/
      space_in_front = $1      
      py_line = "#{space_in_front}time.sleep(#{$2})"
      result << py_line     
      next    
    end
    if py_line =~ /^(\s*)sleep\s+([\d\.]+)\s+#/
      space_in_front = $1      
      py_line = "#{space_in_front}time.sleep(#{$2})"
      result << py_line     
      next    
    end

    
    if py_line =~ /^(\s*)puts\s+(.*)/
      space_in_front = $1      
      py_line = "#{space_in_front}self.puts(#{$2})"
      result << py_line     
      next    
    end
    
    if py_line =~ /^(\s*)visit\s+(.*)/
      space_in_front = $1      
      py_line = "#{space_in_front}self.visit(#{$2})"
      result << py_line     
      next    
    end

    if py_line =~ /^(\s*)visit\((.*)/
      space_in_front = $1      
      py_line = "#{space_in_front}self.visit(#{$2}"
      result << py_line     
      next    
    end

    if py_line.strip == 'load File.dirname(__FILE__) + "/../test_helper.rb"'
      next
    end        
    
    py_line.gsub!(":control", "Keys.CONTROL")
    py_line.gsub!(":end", "Keys.END")
    py_line.gsub!(":down", "Keys.DOWN")
    py_line.gsub!(":up", "Keys.UP")
    py_line.gsub!(":return", "Keys.RETURN")
    py_line.gsub!(":shift", "Keys.SHIIFT")
    py_line.gsub!(":backspace", "Keys.BACKSPACE")
    py_line.gsub!(":tab", "Keys.TAB")
    
    py_line.gsub!("driver.find_element(:name,", "self.driver.find_element(By.NAME,")
    py_line.gsub!("driver.find_element(:id,", "self.driver.find_element(By.ID,")
    py_line.gsub!("driver.find_element(:xpath,", "self.driver.find_element(By.XPATH,")
    py_line.gsub!("driver.find_element(:link_text,", "self.driver.find_element(By.LINK_TEXT,")
    py_line.gsub!("driver.find_element(:partial_link_text,", "self.driver.find_element(By.PARTIAL_LINK_TEXT,")
    py_line.gsub!("driver.find_element(:tag_name,", "self.driver.find_element(By.TAG_NAME,")
    py_line.gsub!("driver.find_element(:class_name,", "self.driver.find_element(By.CLASS_NAME,")
    py_line.gsub!("driver.find_element(:class,", "self.driver.find_element(By.CLASS_NAME,")
    py_line.gsub!("driver.find_element(:css,", "self.driver.find_element(By.CSS_SELECTOR,")


    py_line.gsub!(").select_by(:text, ", ").select_by_visible_text(")
    py_line.gsub!("Selenium::WebDriver::Support::Select.new(", "Select(")    
    py_line.gsub!(".attribute(", ".get_attribute(")
    py_line.gsub!("puts(", "self.puts(")    
    
    # special instance variables
    py_line.gsub!("@server_env", "self.server_env")    
    
    
    # expect(xxx).to include(yyy)
    if py_line =~ /^(\s*)expect\((.*)\)\.to\s+include\((.*)\)\s*/
      space_in_front = $1
      actual = $2
      expected = $3
      py_line = "#{space_in_front}self.assertIn(#{expected}, #{actual})"
      result << py_line    
      next      
    end
    
    # expect(xxx).to eq(yyy)
    if py_line =~ /^(\s*)expect\((.*)\)\.to\s+eq\((.*)\)\s*/
      space_in_front = $1
      actual = $2
      expected = $3
      py_line = "#{space_in_front}self.assertEqual(#{expected}, #{actual})"
      result << py_line  
      next        
    end
    
    if py_line.strip.start_with?("self.driver.find_element") && py_line.strip.end_with?(".click")
      py_line += "()"
    end
    
    result << py_line        
  end

  # TODO  
  # @classmethod
  # self.driver => cls.driver
  
  target_test_content = result.join("\n")
  # puts target_page_content

  FileUtils.mkdir_p(target_project_test_dir) unless Dir.exist?(target_project_test_dir)
  target_file = File.join( target_project_test_dir, File.basename(test_file_path).gsub("_spec.rb", "_test.py"))
  File.open(target_file, "w").write(target_test_content)  
end


if ARGV.count != 2
  exit(-1)
end

in_project_dir = File.expand_path ARGV[0]
target_project_dir = File.expand_path ARGV[1]

python_page_file_names = ["abstract_page.py"]

Dir.entries("#{in_project_dir}/pages").each do |page_file_name|
  next if page_file_name == "." || page_file_name == ".."
  next if page_file_name == "abstract_page.rb"
  python_page_file_names << page_file_name.gsub(".rb", ".py")
  file_path = File.expand_path(File.join("#{in_project_dir}/pages", page_file_name))

  page_class_name = convert_page_file(file_path, target_project_dir)
  @page_class_hash[page_file_name.gsub(".rb", "")] = page_class_name
end


Dir.entries("#{in_project_dir}/spec").each do |test_file_name|
  next if test_file_name == "." || test_file_name == ".."
  next if test_file_name == "spec_helper.rb"
  next if test_file_name == "debugging_spec.rb"
  next unless test_file_name.include?("_spec.rb")
  
  # python_test_file_names << page_file_name.gsub(".rb", ".py")
  file_path = File.expand_path(File.join("#{in_project_dir}/spec", test_file_name))
  convert_test_file(file_path, File.join(target_project_dir, "tests"))
end


page_init_py_file = File.join(target_project_dir, "pages", "__init__.py")
File.open(page_init_py_file, "w").puts("__all__ = " + python_page_file_names.inspect)
